@@ -3,7 +3,7 @@ import json
 import logging
 import time
 import timeit
-from typing import Any, List
+from typing import Any, Dict, List, Optional
 
 import boto3
 import s3path
@@ -15,6 +15,29 @@ log = logging.getLogger(__name__)
 # Ref: https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/glue.html
 
 
+def create_database(**kwargs: Any) -> None:
+    """Safely create the specified AWS Glue database.
+
+    At minimum the `Name` keyword argument is required.
+
+    :param kwargs: These are forwarded to `create_database` as its `CatalogId` and `DatabaseInput` parameters.
+    """
+    name = kwargs["Name"]
+    call_kwargs = {"DatabaseInput": {k: v for k, v in kwargs.items() if k != "CatalogId"}}
+    assert call_kwargs
+    if catalog_id := kwargs.get("CatalogId"):
+        call_kwargs["CatalogId"] = catalog_id
+    client = boto3.client("glue")
+
+    try:
+        response = client.create_database(**call_kwargs)
+    except client.exceptions.AlreadyExistsException:
+        log.info(f"AWS Glue has a database named {name}.")
+    else:
+        assert response["ResponseMetadata"]["HTTPStatusCode"] == 200
+        log.info(f"Created AWS Glue database {name}.")
+
+
 def delete_database(**kwargs: Any) -> None:
     """Safely delete the specified AWS Glue database.
 
@@ -23,10 +46,13 @@ def delete_database(**kwargs: Any) -> None:
     :param kwargs: These are forwarded to `delete_database`.
     """
     name = kwargs["Name"]
+    call_kwargs = kwargs.copy()
+    if not call_kwargs.get("CatalogId"):
+        call_kwargs.pop("CatalogId", None)  # Removes key if having None value.
     client = boto3.client("glue")
 
     try:
-        response = client.delete_database(**kwargs)
+        response = client.delete_database(**call_kwargs)
     except client.exceptions.EntityNotFoundException:
         log.info(f"AWS Glue does not have a database named {name}.")
     else:
@@ -50,15 +76,19 @@ def ensure_and_run_crawler(**kwargs: Any) -> List[dict]:
     return tables
 
 
-def ensure_and_run_s3_tables_crawler(crawler: str, s3_path: s3path.S3Path, **kwargs: Any) -> List[dict]:
+def ensure_and_run_s3_tables_crawler(crawler: str, s3_path: s3path.S3Path, s3_exclusions: Optional[List[str]] = None, **kwargs: Any) -> List[dict]:
     """Return the list of database tables after ensuring and running the specified AWS Glue crawler to crawl the top-level tables in the given S3 path.
 
+    :param s3_exclusions: These are forwarded for each S3 path.
     :param kwargs: These are indirectly forwarded to `create_crawler`.
     """
     # Ref: https://stackoverflow.com/a/67185483/
     assert all(k not in kwargs for k in ("Name", "Targets"))
 
-    s3_crawl_targets = [{"Path": path_to_uri(p)} for p in s3_path.glob("*")]  # For correct crawling, each table's path is independently included.
+    s3_crawl_extra_kwargs = {"Exclusions": s3_exclusions} if s3_exclusions else {}
+    s3_crawl_targets: List[Dict[str, Any]] = [
+        {"Path": path_to_uri(p), **s3_crawl_extra_kwargs} for p in s3_path.glob("*")
+    ]  # For correct crawling, each table's path is independently included.
     assert s3_crawl_targets
     log.info(f"{len(s3_crawl_targets)} S3 paths are to be crawled, the first of which is {s3_crawl_targets[0]['Path']}.")
 
@@ -109,6 +139,19 @@ def get_tables(**kwargs: Any) -> List[dict]:
     return boto3.client("glue").get_paginator("get_tables").paginate(**kwargs).build_full_result()["TableList"]
 
 
+def recreate_db(**kwargs: Any) -> None:
+    """Safely delete and create the specified AWS Glue database.
+
+    At minimum the `Name` keyword argument is required.
+
+    :param kwargs: These are forwarded.
+    The `CatalogId` (optional) and `Name` (required) keyword arguments are forwarded to `delete_database`.
+    All kwargs are forwarded to `create_database` as its `CatalogId` and `DatabaseInput` parameters.
+    """
+    delete_database(CatalogId=kwargs.get("CatalogId"), Name=kwargs["Name"])  # Note: Additional kwargs, such as for `create_database`, are not relevant.
+    create_database(**kwargs)
+
+
 def run_crawler(crawler: str, *, timeout_minutes: int = 120, retry_seconds: int = 5) -> None:
     """Run the specified AWS Glue crawler, waiting until completion."""
     # Ref: https://stackoverflow.com/a/66072347/
